﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HeuristicLab.Services.Hive.DataAccess;

namespace HeuristicLab.Services.Hive.Statistics.Controllers {
  public class AdminDataController : Controller {

    public JsonResult UserList()
    {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString))
      {
        var allUsers = (from users in db.DimUsers
                            select new { users.Name })
                            .ToList();

        List<string> UserNames = new List<string>();
        allUsers.ForEach(u => UserNames.Add(u.Name));

        return Json(UserNames, JsonRequestBehavior.AllowGet);
      }
    }

    public JsonResult UserTaskStats(string userName, DateTime start, DateTime end)
    {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString))
      {
        var taskStats = (from tasks in db.FactTasks
                        join jobs in db.DimJobs
                          on tasks.JobId equals jobs.JobId
                        where jobs.UserName == userName &&
                        tasks.StartTime >= start &&
                        tasks.EndTime < end
                        select new { tasks.TotalWaitingTime, tasks.TotalTransferTime, tasks.TotalRuntime, tasks.NumCalculationRuns, 
                          tasks.CoresRequired, tasks.MemoryRequired, tasks.TaskSize, tasks.ResultSize})
                          .ToList();

        List<KeyValuePair<string, double>> FullStats = new List<KeyValuePair<string, double>>();
        FullStats.Add(new KeyValuePair<string, double>("Total Waiting Time",taskStats.Sum(w => w.TotalWaitingTime)));
        FullStats.Add(new KeyValuePair<string, double>("Total Transfer Time", taskStats.Sum(t => t.TotalTransferTime)));
        FullStats.Add(new KeyValuePair<string, double>("Total Runtime", taskStats.Sum(r => r.TotalRuntime)));
        FullStats.Add(new KeyValuePair<string, double>("Number Calculation Runs", taskStats.Sum(n => n.NumCalculationRuns)));
        FullStats.Add(new KeyValuePair<string, double>("Cores Required", taskStats.Sum(c => c.CoresRequired)));
        FullStats.Add(new KeyValuePair<string, double>("Memory Required", taskStats.Sum(m => m.MemoryRequired)));
        FullStats.Add(new KeyValuePair<string, double>("Task Size", taskStats.Sum(ts => ts.TaskSize)));
        FullStats.Add(new KeyValuePair<string, double>("Result Size", taskStats.Sum(rs => Convert.ToDouble(rs.ResultSize))));

        return Json(FullStats, JsonRequestBehavior.AllowGet);
      }
    }

    public JsonResult AllTaskStats(DateTime start, DateTime end)
    {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString))
      {
        var taskStats = (from tasks in db.FactTasks
                         where tasks.StartTime >= start &&
                         tasks.EndTime < end
                         select new
                         {
                           tasks.TotalWaitingTime,
                           tasks.TotalTransferTime,
                           tasks.TotalRuntime,
                           tasks.NumCalculationRuns,
                           tasks.CoresRequired,
                           tasks.MemoryRequired,
                           tasks.TaskSize,
                           tasks.ResultSize
                         })
                          .ToList();

        List<double> FullStats = new List<double>();
        FullStats.Add(taskStats.Sum(w => w.TotalWaitingTime));
        FullStats.Add(taskStats.Sum(t => t.TotalTransferTime));
        FullStats.Add(taskStats.Sum(r => r.TotalRuntime));
        FullStats.Add(taskStats.Sum(n => n.NumCalculationRuns));
        FullStats.Add(taskStats.Sum(c => c.CoresRequired));
        FullStats.Add(taskStats.Sum(m => m.MemoryRequired));
        FullStats.Add(taskStats.Sum(ts => ts.TaskSize));
        FullStats.Add(taskStats.Sum(rs => Convert.ToDouble(rs.ResultSize)));

        return Json(FullStats, JsonRequestBehavior.AllowGet);
      }
    }
  }
}
